@@ -1,6 +1,15 @@
 #ifndef OSM2PGSQL_DB_COPY_MGR_HPP
 #define OSM2PGSQL_DB_COPY_MGR_HPP
 
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <cassert>
 #include <memory>
 #include <string>
@@ -27,7 +36,7 @@ public:
      */
     void new_line(std::shared_ptr<db_target_descr_t> const &table)
     {
-        if (!m_current || !m_current->target->same_copy_target(*table.get())) {
+        if (!m_current || !m_current->target->same_copy_target(*table)) {
             if (m_current) {
                 m_processor->add_buffer(std::move(m_current));
             }
