@@ -1,8 +1,15 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 /*
  * Dirty tile list generation
  *
- * Steve Hill <steve@nexusuk.org>
- *
  * Please refer to the OpenPisteMap expire_tiles.py script for a demonstration
  * of how to make use of the output:
  * http://subversion.nexusuk.org/projects/openpistemap/trunk/scripts/expire_tiles.py
@@ -16,6 +23,7 @@
 
 #include "expire-tiles.hpp"
 #include "format.hpp"
+#include "logging.hpp"
 #include "options.hpp"
 #include "reprojection.hpp"
 #include "table.hpp"
@@ -31,10 +39,9 @@ tile_output_t::tile_output_t(char const *filename)
 : outfile(fopen(filename, "a"))
 {
     if (outfile == nullptr) {
-        fmt::print(stderr,
-                   "Failed to open expired tiles file ({}).  Tile expiry "
-                   "list will not be written!\n",
-                   std::strerror(errno));
+        log_warn("Failed to open expired tiles file ({}).  Tile expiry "
+                 "list will not be written!",
+                 std::strerror(errno));
     }
 }
 
@@ -52,10 +59,6 @@ void tile_output_t::output_dirty_tile(uint32_t x, uint32_t y, uint32_t zoom)
     }
 
     fmt::print(outfile, "{}/{}/{}\n", zoom, x, y);
-    ++outcount;
-    if (outcount % 1000 == 0) {
-        fmt::print(stderr, "\rWriting dirty tile list ({}K)", outcount / 1000);
-    }
 }
 
 void expire_tiles::output_and_destroy(char const *filename, uint32_t minzoom)
@@ -122,6 +125,16 @@ int expire_tiles::normalise_tile_x_coord(int x)
     return x;
 }
 
+void expire_tiles::coords_to_tile(double lon, double lat, double *tilex,
+                                  double *tiley)
+{
+    auto const c =
+        projection->target_to_tile(osmium::geom::Coordinates{lon, lat});
+
+    *tilex = map_width * (0.5 + c.x / EARTH_CIRCUMFERENCE);
+    *tiley = map_width * (0.5 - c.y / EARTH_CIRCUMFERENCE);
+}
+
 /*
  * Expire tiles that a line crosses
  */
@@ -133,8 +146,8 @@ void expire_tiles::from_line(double lon_a, double lat_a, double lon_b,
     double tile_x_b;
     double tile_y_b;
 
-    projection->coords_to_tile(&tile_x_a, &tile_y_a, lon_a, lat_a, map_width);
-    projection->coords_to_tile(&tile_x_b, &tile_y_b, lon_b, lat_b, map_width);
+    coords_to_tile(lon_a, lat_a, &tile_x_a, &tile_y_a);
+    coords_to_tile(lon_b, lat_b, &tile_x_b, &tile_y_b);
 
     if (tile_x_a > tile_x_b) {
         /* We always want the line to go from left to right - swap the ends if it doesn't */
@@ -224,10 +237,10 @@ int expire_tiles::from_bbox(double min_lon, double min_lat, double max_lon,
     /* Convert the box's Mercator coordinates into tile coordinates */
     double tmp_x;
     double tmp_y;
-    projection->coords_to_tile(&tmp_x, &tmp_y, min_lon, max_lat, map_width);
+    coords_to_tile(min_lon, max_lat, &tmp_x, &tmp_y);
     int min_tile_x = tmp_x - TILE_EXPIRY_LEEWAY;
     int min_tile_y = tmp_y - TILE_EXPIRY_LEEWAY;
-    projection->coords_to_tile(&tmp_x, &tmp_y, max_lon, min_lat, map_width);
+    coords_to_tile(max_lon, min_lat, &tmp_x, &tmp_y);
     int max_tile_x = tmp_x + TILE_EXPIRY_LEEWAY;
     int max_tile_y = tmp_y + TILE_EXPIRY_LEEWAY;
     if (min_tile_x < 0) {
@@ -287,8 +300,7 @@ void expire_tiles::from_wkb(char const *wkb, osmid_t osm_id)
         break;
     }
     default:
-        fmt::print(stderr, "OSM id {}: Unknown geometry type, cannot expire.\n",
-                   osm_id);
+        log_warn("OSM id {}: Unknown geometry type, cannot expire.", osm_id);
     }
 }
 
@@ -349,10 +361,9 @@ void expire_tiles::from_wkb_polygon(ewkb::parser_t *wkb, osmid_t osm_id)
 
     if (from_bbox(min.x, min.y, max.x, max.y)) {
         /* Bounding box too big - just expire tiles on the line */
-        fmt::print(stderr,
-                   "\rLarge polygon ({:.0f} x {:.0f} metres, OSM ID {}"
-                   ") - only expiring perimeter\n",
-                   max.x - min.x, max.y - min.y, osm_id);
+        log_debug("Large polygon ({:.0f} x {:.0f} metres, OSM ID {})"
+                  " - only expiring perimeter",
+                  max.x - min.x, max.y - min.y, osm_id);
         wkb->rewind(start);
         for (unsigned ring = 0; ring < num_rings; ++ring) {
             from_wkb_line(wkb);
