@@ -1,9 +1,19 @@
 #ifndef OSM2PGSQL_EXPIRE_TILES_HPP
 #define OSM2PGSQL_EXPIRE_TILES_HPP
 
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <memory>
 #include <unordered_set>
 
+#include "logging.hpp"
 #include "osmtypes.hpp"
 #include "pgsql.hpp"
 
@@ -19,9 +29,8 @@ class parser_t;
  */
 struct xy_coord_t
 {
-    uint32_t x;
-    uint32_t y;
-    xy_coord_t() : x(0), y(0) {}
+    uint32_t x = 0;
+    uint32_t y = 0;
 };
 
 /**
@@ -30,10 +39,9 @@ struct xy_coord_t
 class tile_output_t
 {
     FILE *outfile;
-    uint32_t outcount = 0;
 
 public:
-    tile_output_t(char const *filename);
+    explicit tile_output_t(char const *filename);
 
     ~tile_output_t();
 
@@ -93,6 +101,7 @@ struct expire_tiles
          * last_quadkey is initialized with a value which is not expected to exist
          * (larger than largest possible quadkey). */
         uint64_t last_quadkey = 1ULL << (2 * maxzoom);
+        std::size_t count = 0;
         for (std::vector<uint64_t>::const_iterator it = tiles_maxzoom.cbegin();
              it != tiles_maxzoom.cend(); ++it) {
             for (uint32_t dz = 0; dz <= maxzoom - minzoom; ++dz) {
@@ -107,9 +116,11 @@ struct expire_tiles
                 }
                 xy_coord_t xy = quadkey_to_xy(qt_current, maxzoom - dz);
                 output_writer.output_dirty_tile(xy.x, xy.y, maxzoom - dz);
+                ++count;
             }
             last_quadkey = *it;
         }
+        log_info("Wrote {} entries to expired tiles list", count);
     }
 
     /**
@@ -142,6 +153,12 @@ struct expire_tiles
     static xy_coord_t quadkey_to_xy(uint64_t quadkey, uint32_t zoom);
 
 private:
+
+    /**
+     * Converts from target coordinates to tile coordinates.
+     */
+    void coords_to_tile(double lon, double lat, double *tilex, double *tiley);
+
     /**
      * Expire a single tile.
      *
