@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include "flex-table-column.hpp"
 
 #include <algorithm>
@@ -13,31 +22,31 @@ struct column_type_lookup
     table_column_type type;
 };
 
-static column_type_lookup const column_types[] = {
-    {"sql", table_column_type::sql},
-    {"text", table_column_type::text},
-    {"boolean", table_column_type::boolean},
-    {"bool", table_column_type::boolean},
-    {"int2", table_column_type::int2},
-    {"smallint", table_column_type::int2},
-    {"int4", table_column_type::int4},
-    {"int", table_column_type::int4},
-    {"integer", table_column_type::int4},
-    {"int8", table_column_type::int8},
-    {"bigint", table_column_type::int8},
-    {"real", table_column_type::real},
-    {"hstore", table_column_type::hstore},
-    {"direction", table_column_type::direction},
-    {"geometry", table_column_type::geometry},
-    {"point", table_column_type::point},
-    {"linestring", table_column_type::linestring},
-    {"polygon", table_column_type::polygon},
-    {"multipoint", table_column_type::multipoint},
-    {"multilinestring", table_column_type::multilinestring},
-    {"multipolygon", table_column_type::multipolygon},
-    {"area", table_column_type::area},
-    {"id_type", table_column_type::id_type},
-    {"id_num", table_column_type::id_num}};
+static std::array<column_type_lookup, 24> const column_types = {
+    {{"sql", table_column_type::sql},
+     {"text", table_column_type::text},
+     {"boolean", table_column_type::boolean},
+     {"bool", table_column_type::boolean},
+     {"int2", table_column_type::int2},
+     {"smallint", table_column_type::int2},
+     {"int4", table_column_type::int4},
+     {"int", table_column_type::int4},
+     {"integer", table_column_type::int4},
+     {"int8", table_column_type::int8},
+     {"bigint", table_column_type::int8},
+     {"real", table_column_type::real},
+     {"hstore", table_column_type::hstore},
+     {"direction", table_column_type::direction},
+     {"geometry", table_column_type::geometry},
+     {"point", table_column_type::point},
+     {"linestring", table_column_type::linestring},
+     {"polygon", table_column_type::polygon},
+     {"multipoint", table_column_type::multipoint},
+     {"multilinestring", table_column_type::multilinestring},
+     {"multipolygon", table_column_type::multipolygon},
+     {"area", table_column_type::area},
+     {"id_type", table_column_type::id_type},
+     {"id_num", table_column_type::id_num}}};
 
 static table_column_type
 get_column_type_from_string(std::string const &type) noexcept
@@ -75,7 +84,34 @@ flex_table_column_t::flex_table_column_t(std::string name,
   m_type(get_column_type_from_string(m_type_name))
 {}
 
-std::string flex_table_column_t::sql_type_name(int srid) const
+void flex_table_column_t::set_projection(char const *projection)
+{
+    if (!projection || *projection == '\0') {
+        return;
+    }
+
+    auto const proj = lowercase(projection);
+
+    if (proj == "merc" || proj == "mercator") {
+        m_srid = 3857;
+        return;
+    }
+
+    if (proj == "latlong" || proj == "latlon" || proj == "wgs84") {
+        m_srid = 4326;
+        return;
+    }
+
+    char *end = nullptr;
+    m_srid = static_cast<int>(std::strtoul(projection, &end, 10));
+
+    if (*end != '\0') {
+        throw std::runtime_error{
+            "Unknown projection: '{}'."_format(projection)};
+    }
+}
+
+std::string flex_table_column_t::sql_type_name() const
 {
     switch (m_type) {
     case table_column_type::sql:
@@ -97,19 +133,19 @@ std::string flex_table_column_t::sql_type_name(int srid) const
     case table_column_type::direction:
         return "int2";
     case table_column_type::geometry:
-        return "Geometry(GEOMETRY, {})"_format(srid);
+        return "Geometry(GEOMETRY, {})"_format(m_srid);
     case table_column_type::point:
-        return "Geometry(POINT, {})"_format(srid);
+        return "Geometry(POINT, {})"_format(m_srid);
     case table_column_type::linestring:
-        return "Geometry(LINESTRING, {})"_format(srid);
+        return "Geometry(LINESTRING, {})"_format(m_srid);
     case table_column_type::polygon:
-        return "Geometry(POLYGON, {})"_format(srid);
+        return "Geometry(POLYGON, {})"_format(m_srid);
     case table_column_type::multipoint:
-        return "Geometry(MULTIPOINT, {})"_format(srid);
+        return "Geometry(MULTIPOINT, {})"_format(m_srid);
     case table_column_type::multilinestring:
-        return "Geometry(MULTILINESTRING, {})"_format(srid);
+        return "Geometry(MULTILINESTRING, {})"_format(m_srid);
     case table_column_type::multipolygon:
-        return "Geometry(MULTIPOLYGON, {})"_format(srid);
+        return "Geometry(MULTIPOLYGON, {})"_format(m_srid);
     case table_column_type::area:
         return "real";
     case table_column_type::id_type:
@@ -117,7 +153,7 @@ std::string flex_table_column_t::sql_type_name(int srid) const
     case table_column_type::id_num:
         return "int8";
     }
-    throw std::runtime_error{"Unknown column type"};
+    throw std::runtime_error{"Unknown column type."};
 }
 
 std::string flex_table_column_t::sql_modifiers() const
@@ -135,7 +171,7 @@ std::string flex_table_column_t::sql_modifiers() const
     return modifiers;
 }
 
-std::string flex_table_column_t::sql_create(int srid) const
+std::string flex_table_column_t::sql_create() const
 {
-    return "\"{}\" {} {},"_format(m_name, sql_type_name(srid), sql_modifiers());
+    return "\"{}\" {} {},"_format(m_name, sql_type_name(), sql_modifiers());
 }
