@@ -1,7 +1,17 @@
 #ifndef OSM2PGSQL_FLEX_TABLE_COLUMN_HPP
 #define OSM2PGSQL_FLEX_TABLE_COLUMN_HPP
 
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include "format.hpp"
+#include "reprojection.hpp"
 
 #include <cstdint>
 #include <string>
@@ -85,9 +95,13 @@ public:
 
     void set_create_only(bool value = true) noexcept { m_create_only = value; }
 
-    std::string sql_type_name(int srid) const;
+    void set_projection(char const *projection);
+
+    std::string sql_type_name() const;
     std::string sql_modifiers() const;
-    std::string sql_create(int srid) const;
+    std::string sql_create() const;
+
+    int srid() const noexcept { return m_srid; }
 
 private:
     /// The name of the database table column.
@@ -105,6 +119,12 @@ private:
      */
     table_column_type m_type;
 
+    /**
+     * For geometry and area columns only: The projection SRID. Default is
+     * web mercator.
+     */
+    int m_srid = 3857;
+
     /// NOT NULL constraint
     bool m_not_null = false;
 
