@@ -1,6 +1,15 @@
 #ifndef OSM2PGSQL_GEOM_TRANSFORM_HPP
 #define OSM2PGSQL_GEOM_TRANSFORM_HPP
 
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include "flex-table-column.hpp"
 #include "osmium-builder.hpp"
 
@@ -32,19 +41,22 @@ public:
 
     virtual geom::osmium_builder_t::wkbs_t
     run(geom::osmium_builder_t * /*builder*/,
+        table_column_type /*target_geom_type*/,
         osmium::Node const & /*node*/) const
     {
         return {};
     }
 
     virtual geom::osmium_builder_t::wkbs_t
-    run(geom::osmium_builder_t * /*builder*/, osmium::Way * /*way*/) const
+    run(geom::osmium_builder_t * /*builder*/,
+        table_column_type /*target_geom_type*/, osmium::Way * /*way*/) const
     {
         return {};
     }
 
     virtual geom::osmium_builder_t::wkbs_t
     run(geom::osmium_builder_t * /*builder*/,
+        table_column_type /*target_geom_type*/,
         osmium::Relation const & /*relation*/,
         osmium::memory::Buffer const & /*buffer*/) const
     {
@@ -60,6 +72,7 @@ public:
         noexcept override;
 
     geom::osmium_builder_t::wkbs_t run(geom::osmium_builder_t *builder,
+                                       table_column_type target_geom_type,
                                        osmium::Node const &node) const override;
 
 }; // class geom_transform_point_t
@@ -73,10 +86,12 @@ public:
         noexcept override;
 
     geom::osmium_builder_t::wkbs_t run(geom::osmium_builder_t *builder,
+                                       table_column_type target_geom_type,
                                        osmium::Way *way) const override;
 
     geom::osmium_builder_t::wkbs_t
-    run(geom::osmium_builder_t *builder, osmium::Relation const &relation,
+    run(geom::osmium_builder_t *builder, table_column_type target_geom_type,
+        osmium::Relation const &relation,
         osmium::memory::Buffer const &buffer) const override;
 
 private:
@@ -93,14 +108,16 @@ public:
         noexcept override;
 
     geom::osmium_builder_t::wkbs_t run(geom::osmium_builder_t *builder,
+                                       table_column_type target_geom_type,
                                        osmium::Way *way) const override;
 
     geom::osmium_builder_t::wkbs_t
-    run(geom::osmium_builder_t *builder, osmium::Relation const &relation,
+    run(geom::osmium_builder_t *builder, table_column_type target_geom_type,
+        osmium::Relation const &relation,
         osmium::memory::Buffer const &buffer) const override;
 
 private:
-    bool m_multi = false;
+    bool m_multi = true;
 
 }; // class geom_transform_area_t
 
