@@ -1,6 +1,15 @@
 #ifndef OSM2PGSQL_MIDDLE_RAM_HPP
 #define OSM2PGSQL_MIDDLE_RAM_HPP
 
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 /* Implements the mid-layer processing for osm2pgsql
  * using data structures in RAM.
  *
@@ -75,7 +84,7 @@ public:
         assert(block < m_blocks.size());
 
         if (!m_blocks[block]) {
-            return 0;
+            return nullptr;
         }
 
         return m_blocks[block]->get(id2offset(id));
@@ -91,29 +100,25 @@ public:
 
 struct middle_ram_t : public middle_t, public middle_query_t
 {
-    middle_ram_t(options_t const *options);
-    virtual ~middle_ram_t() noexcept = default;
+    explicit middle_ram_t(options_t const *options);
+    ~middle_ram_t() noexcept override = default;
 
     void start() override {}
     void stop(thread_pool_t &pool) override;
-    void analyze() override {}
-    void commit() override {}
 
-    void node_set(osmium::Node const &node) override;
+    void node(osmium::Node const &node) override;
+    void way(osmium::Way const &way) override;
+    void relation(osmium::Relation const &rel) override;
+
     size_t nodes_get_list(osmium::WayNodeList *nodes) const override;
 
-    void way_set(osmium::Way const &way) override;
-    bool way_get(osmid_t id, osmium::memory::Buffer &buffer) const override;
+    bool way_get(osmid_t id, osmium::memory::Buffer *buffer) const override;
+
     size_t rel_way_members_get(osmium::Relation const &rel, rolelist_t *roles,
-                               osmium::memory::Buffer &buffer) const override;
+                               osmium::memory::Buffer *buffer) const override;
 
     bool relation_get(osmid_t id,
-                      osmium::memory::Buffer &buffer) const override;
-    void relation_set(osmium::Relation const &rel) override;
-
-    void flush() override {}
-
-    idlist_t relations_using_way(osmid_t way_id) const override;
+                      osmium::memory::Buffer *buffer) const override;
 
     std::shared_ptr<middle_query_t> get_query_instance() override;
 
