@@ -1,9 +1,19 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 /* Implements a node cache in ram, for the middle layers to use.
  * It uses two different storage methods, one optimized for dense
  * nodes (with respect to id) and the other for sparse representations.
 */
 
 #include "format.hpp"
+#include "logging.hpp"
 #include "node-ram-cache.hpp"
 #include "osmtypes.hpp"
 #include "util.hpp"
@@ -95,7 +105,7 @@ void node_ram_cache::set_sparse(osmid_t id, osmium::Location location)
             }
             throw std::runtime_error{
                 "Node cache size is too small to fit all nodes. Please "
-                "increase cache size\n"};
+                "increase cache size."};
         }
     }
     maxSparseId = id;
@@ -174,8 +184,7 @@ void node_ram_cache::set_dense(osmid_t id, osmium::Location location)
             blocks[block].reset_used();
             blocks[block].block_offset = block;
             if (!blocks[block].nodes) {
-                fprintf(stderr, "Error allocating nodes\n");
-                util::exit_nicely();
+                throw std::runtime_error{"Error allocating nodes."};
             }
             queue[usedBlocks] = &blocks[block];
             ++usedBlocks;
@@ -189,10 +198,9 @@ void node_ram_cache::set_dense(osmid_t id, osmium::Location location)
             }
         } else {
             if ((allocStrategy & ALLOC_LOSSY) == 0) {
-                fprintf(stderr,
-                        "\nNode cache size is too small to fit all nodes. "
-                        "Please increase cache size\n");
-                util::exit_nicely();
+                throw std::runtime_error{
+                    "Node cache size is too small to fit all nodes. "
+                    "Please increase cache size."};
             }
             /* We've reached the maximum number of blocks, so now we push the
              * current head of the tree down to the right level to restore the
@@ -235,23 +243,15 @@ void node_ram_cache::set_dense(osmid_t id, osmium::Location location)
          * will break the invariant. However, it will work fine if all the
          * nodes come in numerical order, which is the common case */
 
-        int expectedpos;
+        int expectedpos = 0;
         if ((usedBlocks < maxBlocks) && (cacheUsed < cacheSize)) {
             expectedpos = usedBlocks - 1;
-        } else {
-            expectedpos = 0;
         }
 
         if (queue[expectedpos] != &blocks[block]) {
-            if (m_warn_node_order) {
-                fmt::print(stderr,
-                           "WARNING: Found out of order node {}"
-                           " ({},{}) - this will impact the cache efficiency\n",
-                           id, block, offset);
-
-                // Only warn once
-                m_warn_node_order = false;
-            }
+            // This can only happen if the node is out of order which it
+            // should never be because of earlier checks for that.
+            // Ignore the node.
             return;
         }
     }
@@ -310,13 +310,12 @@ node_ram_cache::node_ram_cache(int strategy, int cacheSizeMB)
     maxSparseTuples = (cacheSize / sizeof(ramNodeID)) + 1;
 
     if ((allocStrategy & ALLOC_DENSE) > 0) {
-        fprintf(stderr, "Allocating memory for dense node cache\n");
+        log_debug("Allocating memory for dense node cache");
         blocks = (ramNodeBlock *)calloc(num_blocks(), sizeof(ramNodeBlock));
         if (!blocks) {
-            fprintf(stderr,
-                    "Out of memory for node cache dense index, try using "
-                    "\"--cache-strategy sparse\" instead \n");
-            util::exit_nicely();
+            throw std::runtime_error{
+                "Out of memory for node cache dense index, try using "
+                "\"--cache-strategy sparse\" instead."};
         }
         queue = (ramNodeBlock **)calloc(maxBlocks, sizeof(ramNodeBlock *));
         /* Use this method of allocation if virtual memory is limited,
@@ -324,20 +323,17 @@ node_ram_cache::node_ram_cache(int strategy, int cacheSizeMB)
          * once it is needed.
          */
         if ((allocStrategy & ALLOC_DENSE_CHUNK) > 0) {
-            fprintf(stderr,
-                    "Allocating dense node cache in block sized chunks\n");
+            log_debug("Allocating dense node cache in block sized chunks");
             if (!queue) {
-                fprintf(stderr, "Out of memory, reduce --cache size\n");
-                util::exit_nicely();
+                throw std::runtime_error{"Out of memory, reduce --cache size."};
             }
         } else {
-            fprintf(stderr, "Allocating dense node cache in one big chunk\n");
+            log_debug("Allocating dense node cache in one big chunk");
             blockCache = (char *)malloc((maxBlocks + 1024) * per_block() *
                                         sizeof(osmium::Location));
             if (!queue || !blockCache) {
-                fprintf(stderr, "Out of memory for dense node cache, reduce "
-                                "--cache size\n");
-                util::exit_nicely();
+                throw std::runtime_error{
+                    "Out of memory for dense node cache, reduce --cache size."};
             }
         }
     }
@@ -351,27 +347,23 @@ node_ram_cache::node_ram_cache(int strategy, int cacheSizeMB)
      */
 
     if ((allocStrategy & ALLOC_SPARSE) > 0) {
-        fprintf(stderr, "Allocating memory for sparse node cache\n");
+        log_debug("Allocating memory for sparse node cache");
         if (!blockCache) {
             sparseBlock =
                 (ramNodeID *)malloc(maxSparseTuples * sizeof(ramNodeID));
         } else {
-            fprintf(stderr, "Sharing dense sparse\n");
+            log_debug("Sharing dense sparse");
             sparseBlock = (ramNodeID *)blockCache;
         }
         if (!sparseBlock) {
-            fprintf(
-                stderr,
-                "Out of memory for sparse node cache, reduce --cache size\n");
-            util::exit_nicely();
+            throw std::runtime_error{
+                "Out of memory for sparse node cache, reduce --cache size."};
         }
     }
 
-    fmt::print(
-        stderr,
-        "Node-cache: cache={}MB, maxblocks={}*{}, allocation method={}\n",
-        (cacheSize >> 20U), maxBlocks, per_block() * sizeof(osmium::Location),
-        allocStrategy);
+    log_info("Node-cache: cache={}MB, maxblocks={}*{}, allocation method={}",
+             (cacheSize >> 20U), maxBlocks,
+             per_block() * sizeof(osmium::Location), allocStrategy);
 }
 
 node_ram_cache::~node_ram_cache()
@@ -380,11 +372,10 @@ node_ram_cache::~node_ram_cache()
         return;
     }
 
-    fmt::print(
-        stderr,
+    log_info(
         "node cache: stored: {}"
         "({:.2f}%), storage efficiency: {:.2f}% (dense blocks: {}, "
-        "sparse nodes: {}), hit rate: {:.2f}%\n",
+        "sparse nodes: {}), hit rate: {:.2f}%",
         storedNodes, totalNodes == 0 ? 0.0f : 100.0f * storedNodes / totalNodes,
         cacheUsed == 0
             ? 0.0f
@@ -420,11 +411,8 @@ void node_ram_cache::set(osmid_t id, osmium::Location location)
 
     if ((id > 0 && id >> BLOCK_SHIFT >> 32U) ||
         (id < 0 && ~id >> BLOCK_SHIFT >> 32U)) {
-        fmt::print(stderr,
-                   "\nAbsolute node IDs must not be larger than {}"
-                   " (got {})\n",
-                   (int64_t)1 << 42U, id);
-        util::exit_nicely();
+        throw std::runtime_error{"Absolute node IDs must not be larger than {}"
+                                 " (got {})."_format(1ULL << 42U, id)};
     }
     ++totalNodes;
     /* if ALLOC_DENSE and ALLOC_SPARSE are set, send it through
@@ -439,7 +427,7 @@ void node_ram_cache::set(osmid_t id, osmium::Location location)
         // Command line options always have ALLOC_DENSE | ALLOC_SPARSE
         throw std::logic_error{
             "Unexpected cache strategy in node_ram_cache::set with "
-            "allocStrategy {}"_format(allocStrategy)};
+            "allocStrategy {}."_format(allocStrategy)};
     }
 }
 
