@@ -1,17 +1,29 @@
 #ifndef OSM2PGSQL_OSMTYPES_HPP
 #define OSM2PGSQL_OSMTYPES_HPP
 
-/* Data types to hold OSM node, segment, way data */
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
 
-// when __cplusplus is defined, we need to define this macro as well
-// to get the print format specifiers in the inttypes.h header.
-#define __STDC_FORMAT_MACROS
-#include <cinttypes>
+/**
+ * \file
+ *
+ * This file is part of osm2pgsql (https://github.com/openstreetmap/osm2pgsql).
+ *
+ * In this file some basic (OSM) data types are defined.
+ */
 
 #include <algorithm>
-#include <cmath>
+#include <cstdint>
+#include <cstring>
 #include <limits>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include <osmium/builder/attr.hpp>
@@ -19,7 +31,6 @@
 #include <osmium/osm.hpp>
 
 using osmid_t = std::int64_t;
-#define strtoosmid strtoll
 
 struct member
 {
@@ -27,20 +38,18 @@ struct member
     osmid_t id;
     std::string role;
 
-    operator osmium::builder::attr::member_type const() const
+    explicit operator osmium::builder::attr::member_type() const
     {
         return osmium::builder::attr::member_type(type, id, role.c_str());
     }
 
-    member(osmium::item_type t, osmid_t i, std::string const &r)
-    : type(t), id(i), role(r)
+    member(osmium::item_type t, osmid_t i, std::string r)
+    : type(t), id(i), role(std::move(r))
     {}
 };
 
 struct memberlist_t : public std::vector<member>
 {
-    memberlist_t() {}
-
     explicit memberlist_t(osmium::RelationMemberList const &list)
     {
         for (auto const &m : list) {
@@ -64,13 +73,14 @@ struct tag_t
     std::string key;
     std::string value;
 
-    operator std::pair<char const *, char const *> const() const
+    operator std::pair<char const *, char const *>() const noexcept
     {
         return std::pair<char const *, char const *>(key.c_str(),
                                                      value.c_str());
     }
 
-    tag_t(std::string const &k, std::string const &v) : key(k), value(v) {}
+    tag_t(std::string k, std::string v) : key(std::move(k)), value(std::move(v))
+    {}
 };
 
 /**
