@@ -1,8 +1,15 @@
-
-#include "config.h"
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
 
 #include "expire-tiles.hpp"
 #include "geom-transform.hpp"
+#include "logging.hpp"
 #include "lua-init.hpp"
 #include "lua-utils.hpp"
 #include "middle.hpp"
@@ -11,7 +18,6 @@
 #include "output-flex.hpp"
 #include "pgsql.hpp"
 #include "reprojection.hpp"
-#include "taginfo-impl.hpp"
 #include "util.hpp"
 #include "version.hpp"
 #include "wkb.hpp"
@@ -87,7 +93,7 @@ prepared_lua_function_t::prepared_lua_function_t(lua_State *lua_state,
         return;
     }
 
-    throw std::runtime_error{"osm2pgsql.{} must be a function"_format(name)};
+    throw std::runtime_error{"osm2pgsql.{} must be a function."_format(name)};
 }
 
 static void push_osm_object_to_lua_stack(lua_State *lua_state,
@@ -275,7 +281,7 @@ void write_integer(db_copy_mgr_t<db_deleter_by_type_and_id_t> *copy_mgr,
         return;
     }
 
-    char *end;
+    char *end = nullptr;
     errno = 0;
     auto const value = std::strtoll(str, &end, 10);
 
@@ -311,6 +317,14 @@ void output_flex_t::write_column(
     db_copy_mgr_t<db_deleter_by_type_and_id_t> *copy_mgr,
     flex_table_column_t const &column)
 {
+    // If there is nothing on the Lua stack, then the Lua function add_row()
+    // was called without a table parameter. In that case this column will
+    // be set to NULL.
+    if (lua_gettop(lua_state()) == 0) {
+        write_null(copy_mgr, column);
+        return;
+    }
+
     lua_getfield(lua_state(), -1, column.name().c_str());
     int const ltype = lua_type(lua_state(), -1);
 
@@ -332,8 +346,9 @@ void output_flex_t::write_column(
         (column.type() == table_column_type::text)) {
         auto const *const str = lua_tolstring(lua_state(), -1, nullptr);
         if (!str) {
-            throw std::runtime_error{"Invalid type '{}' for text column"_format(
-                lua_typename(lua_state(), ltype))};
+            throw std::runtime_error{
+                "Invalid type '{}' for text column."_format(
+                    lua_typename(lua_state(), ltype))};
         }
         copy_mgr->add_column(str);
     } else if (column.type() == table_column_type::boolean) {
@@ -350,7 +365,7 @@ void output_flex_t::write_column(
             break;
         default:
             throw std::runtime_error{
-                "Invalid type '{}' for boolean column"_format(
+                "Invalid type '{}' for boolean column."_format(
                     lua_typename(lua_state(), ltype))};
         }
     } else if (column.type() == table_column_type::int2) {
@@ -369,7 +384,7 @@ void output_flex_t::write_column(
             copy_mgr->add_column(lua_toboolean(lua_state(), -1));
         } else {
             throw std::runtime_error{
-                "Invalid type '{}' for int2 column"_format(
+                "Invalid type '{}' for int2 column."_format(
                     lua_typename(lua_state(), ltype))};
         }
     } else if (column.type() == table_column_type::int4) {
@@ -388,7 +403,7 @@ void output_flex_t::write_column(
             copy_mgr->add_column(lua_toboolean(lua_state(), -1));
         } else {
             throw std::runtime_error{
-                "Invalid type '{}' for int4 column"_format(
+                "Invalid type '{}' for int4 column."_format(
                     lua_typename(lua_state(), ltype))};
         }
     } else if (column.type() == table_column_type::int8) {
@@ -401,7 +416,7 @@ void output_flex_t::write_column(
             copy_mgr->add_column(lua_toboolean(lua_state(), -1));
         } else {
             throw std::runtime_error{
-                "Invalid type '{}' for int8 column"_format(
+                "Invalid type '{}' for int8 column."_format(
                     lua_typename(lua_state(), ltype))};
         }
     } else if (column.type() == table_column_type::real) {
@@ -412,7 +427,7 @@ void output_flex_t::write_column(
                          lua_tolstring(lua_state(), -1, nullptr));
         } else {
             throw std::runtime_error{
-                "Invalid type '{}' for real column"_format(
+                "Invalid type '{}' for real column."_format(
                     lua_typename(lua_state(), ltype))};
         }
     } else if (column.type() == table_column_type::hstore) {
@@ -444,7 +459,7 @@ void output_flex_t::write_column(
             copy_mgr->finish_hash();
         } else {
             throw std::runtime_error{
-                "Invalid type '{}' for hstore column"_format(
+                "Invalid type '{}' for hstore column."_format(
                     lua_typename(lua_state(), ltype))};
         }
     } else if (column.type() == table_column_type::direction) {
@@ -461,12 +476,12 @@ void output_flex_t::write_column(
             break;
         default:
             throw std::runtime_error{
-                "Invalid type '{}' for direction column"_format(
+                "Invalid type '{}' for direction column."_format(
                     lua_typename(lua_state(), ltype))};
         }
     } else {
         throw std::runtime_error{
-            "Column type {} not implemented"_format(column.type())};
+            "Column type {} not implemented."_format(column.type())};
     }
 
     lua_pop(lua_state(), 1);
@@ -474,7 +489,7 @@ void output_flex_t::write_column(
 
 void output_flex_t::write_row(table_connection_t *table_connection,
                               osmium::item_type id_type, osmid_t id,
-                              std::string const &geom)
+                              std::string const &geom, int srid)
 {
     assert(table_connection);
     table_connection->new_line();
@@ -495,12 +510,13 @@ void output_flex_t::write_row(table_connection_t *table_connection,
             if (geom.empty()) {
                 write_null(copy_mgr, column);
             } else {
+                // if srid of the area column is the same as for the geom column
                 double const area =
-                    get_options()->reproject_area
-                        ? ewkb::parser_t(geom).get_area<reprojection>(
-                              get_options()->projection.get())
-                        : ewkb::parser_t(geom)
-                              .get_area<osmium::geom::IdentityProjection>();
+                    column.srid() == srid
+                        ? ewkb::parser_t(geom)
+                              .get_area<osmium::geom::IdentityProjection>()
+                        : ewkb::parser_t(geom).get_area<reprojection>(
+                              reprojection::create_projection(srid).get());
                 copy_mgr->add_column(area);
             }
         } else {
@@ -527,11 +543,11 @@ int output_flex_t::app_get_bbox()
     if (m_calling_context != calling_context::process_node &&
         m_calling_context != calling_context::process_way) {
         throw std::runtime_error{"The function get_bbox() can only be called"
-                                 " from process_node() or process_way()"};
+                                 " from process_node() or process_way()."};
     }
 
     if (lua_gettop(lua_state()) > 1) {
-        throw std::runtime_error{"No parameter(s) needed for get_box()"};
+        throw std::runtime_error{"No parameter(s) needed for get_box()."};
     }
 
     if (m_context_node) {
@@ -566,7 +582,7 @@ static void check_name(std::string const &name, char const *in)
     }
 
     throw std::runtime_error{
-        "Special characters are not allowed in {} names: '{}'"_format(in,
+        "Special characters are not allowed in {} names: '{}'."_format(in,
                                                                       name)};
 }
 
@@ -583,10 +599,10 @@ flex_table_t &output_flex_t::create_flex_table()
                                  });
     if (it != m_tables->cend()) {
         throw std::runtime_error{
-            "Table with that name already exists: '{}'"_format(table_name)};
+            "Table with name '{}' already exists."_format(table_name)};
     }
 
-    m_tables->emplace_back(table_name, get_options()->projection->target_srs());
+    m_tables->emplace_back(table_name);
     auto &new_table = m_tables->back();
 
     lua_pop(lua_state(), 1);
@@ -626,10 +642,9 @@ void output_flex_t::setup_id_columns(flex_table_t *table)
     assert(table);
     lua_getfield(lua_state(), -1, "ids");
     if (lua_type(lua_state(), -1) != LUA_TTABLE) {
-        fmt::print(stderr,
-                   "WARNING! Table '{}' doesn't have an 'ids' column. Updates"
-                   " and expire will not work!\n",
-                   table->name());
+        log_warn("Table '{}' doesn't have an 'ids' column. Updates"
+                 " and expire will not work!",
+                 table->name());
         lua_pop(lua_state(), 1); // ids
         return;
     }
@@ -646,18 +661,20 @@ void output_flex_t::setup_id_columns(flex_table_t *table)
     } else if (type == "area") {
         table->set_id_type(osmium::item_type::area);
     } else if (type == "any") {
-        std::string type_column_name{"osm_type"};
-        lua_getfield(lua_state(), -1, "type_column");
+        table->set_id_type(osmium::item_type::undefined);
+        lua_getfield(lua_state(), -2, "type_column");
         if (lua_isstring(lua_state(), -1)) {
-            type_column_name = lua_tolstring(lua_state(), -1, nullptr);
-            check_name(type_column_name, "column");
+            std::string const column_name =
+                lua_tolstring(lua_state(), -1, nullptr);
+            check_name(column_name, "column");
+            auto &column = table->add_column(column_name, "id_type");
+            column.set_not_null();
+        } else if (!lua_isnil(lua_state(), -1)) {
+            throw std::runtime_error{"type_column must be a string or nil."};
         }
         lua_pop(lua_state(), 1); // type_column
-        auto &column = table->add_column(type_column_name, "id_type");
-        column.set_not_null();
-        table->set_id_type(osmium::item_type::undefined);
     } else {
-        throw std::runtime_error{"Unknown ids type: " + type};
+        throw std::runtime_error{"Unknown ids type: {}."_format(type)};
     }
 
     std::string const name =
@@ -675,7 +692,7 @@ void output_flex_t::setup_flex_table_columns(flex_table_t *table)
     lua_getfield(lua_state(), -1, "columns");
     if (lua_type(lua_state(), -1) != LUA_TTABLE) {
         throw std::runtime_error{
-            "No columns defined for table '{}'"_format(table->name())};
+            "No columns defined for table '{}'."_format(table->name())};
     }
 
     std::size_t num_columns = 0;
@@ -683,11 +700,11 @@ void output_flex_t::setup_flex_table_columns(flex_table_t *table)
     while (lua_next(lua_state(), -2) != 0) {
         if (!lua_isnumber(lua_state(), -2)) {
             throw std::runtime_error{
-                "The 'columns' field must contain an array"};
+                "The 'columns' field must contain an array."};
         }
         if (!lua_istable(lua_state(), -1)) {
             throw std::runtime_error{
-                "The entries in the 'columns' array must be tables"};
+                "The entries in the 'columns' array must be tables."};
         }
 
         char const *const type =
@@ -704,21 +721,34 @@ void output_flex_t::setup_flex_table_columns(flex_table_t *table)
         column.set_create_only(luaX_get_table_bool(
             lua_state(), "create_only", -4, "Entry 'create_only'", false));
 
-        lua_pop(lua_state(), 5); // create_only, not_null, column, type, table
+        lua_getfield(lua_state(), -5, "projection");
+        if (!lua_isnil(lua_state(), -1)) {
+            if (column.is_geometry_column() ||
+                column.type() == table_column_type::area) {
+                column.set_projection(lua_tostring(lua_state(), -1));
+            } else {
+                throw std::runtime_error{
+                    "Projection can only be set on geometry and area columns."};
+            }
+        }
+
+        // stack has: projection, create_only, not_null, column, type, table
+        lua_pop(lua_state(), 6);
         ++num_columns;
     }
 
     if (num_columns == 0) {
         throw std::runtime_error{
-            "No columns defined for table '{}'"_format(table->name())};
+            "No columns defined for table '{}'."_format(table->name())};
     }
 }
 
 int output_flex_t::app_define_table()
 {
     if (m_calling_context != calling_context::main) {
-        throw std::runtime_error{"Database tables have to be defined in the"
-                                 " main Lua code, not in any of the callbacks"};
+        throw std::runtime_error{
+            "Database tables have to be defined in the"
+            " main Lua code, not in any of the callbacks."};
     }
 
     luaL_checktype(lua_state(), 1, LUA_TTABLE);
@@ -742,13 +772,13 @@ static std::size_t table_idx_from_param(lua_State *lua_state)
 
     if (user_data == nullptr || !lua_getmetatable(lua_state, 1)) {
         throw std::runtime_error{
-            "First parameter must be of type osm2pgsql.table"};
+            "First parameter must be of type osm2pgsql.table."};
     }
 
     luaL_getmetatable(lua_state, osm2pgsql_table_name);
     if (!lua_rawequal(lua_state, -1, -2)) {
         throw std::runtime_error{
-            "First parameter must be of type osm2pgsql.table"};
+            "First parameter must be of type osm2pgsql.table."};
     }
     lua_pop(lua_state, 2);
 
@@ -761,7 +791,7 @@ flex_table_t const &output_flex_t::get_table_from_param()
 {
     if (lua_gettop(lua_state()) != 1) {
         throw std::runtime_error{
-            "Need exactly one parameter of type osm2pgsql.table"};
+            "Need exactly one parameter of type osm2pgsql.table."};
     }
 
     auto const &table = m_tables->at(table_idx_from_param(lua_state()));
@@ -790,36 +820,43 @@ int output_flex_t::table_add_row()
         m_calling_context != calling_context::process_relation) {
         throw std::runtime_error{
             "The function add_row() can only be called from the "
-            "process_node/way/relation() functions"};
+            "process_node/way/relation() functions."};
     }
 
-    if (lua_gettop(lua_state()) != 2) {
+    // Params are the table object and an optional Lua table with the contents
+    // for the fields.
+    auto const num_params = lua_gettop(lua_state());
+    if (num_params < 1 || num_params > 2) {
         throw std::runtime_error{
-            "Need two parameters: The osm2pgsql.table and the row data"};
+            "Need two parameters: The osm2pgsql.table and the row data."};
     }
 
     auto &table_connection =
         m_table_connections.at(table_idx_from_param(lua_state()));
-    auto &table = table_connection.table();
-    luaL_checktype(lua_state(), 2, LUA_TTABLE);
+    auto const &table = table_connection.table();
+
+    // It there is a second parameter, it must be a Lua table.
+    if (num_params == 2) {
+        luaL_checktype(lua_state(), 2, LUA_TTABLE);
+    }
     lua_remove(lua_state(), 1);
 
     if (m_context_node) {
         if (!table.matches_type(osmium::item_type::node)) {
             throw std::runtime_error{
-                "Trying to add node to table '{}'"_format(table.name())};
+                "Trying to add node to table '{}'."_format(table.name())};
         }
         add_row(&table_connection, *m_context_node);
     } else if (m_context_way) {
         if (!table.matches_type(osmium::item_type::way)) {
             throw std::runtime_error{
-                "Trying to add way to table '{}'"_format(table.name())};
+                "Trying to add way to table '{}'."_format(table.name())};
         }
         add_row(&table_connection, *m_context_way);
     } else if (m_context_relation) {
         if (!table.matches_type(osmium::item_type::relation)) {
             throw std::runtime_error{
-                "Trying to add relation to table '{}'"_format(table.name())};
+                "Trying to add relation to table '{}'."_format(table.name())};
         }
         add_row(&table_connection, *m_context_relation);
     }
@@ -841,7 +878,7 @@ int output_flex_t::table_columns()
         luaX_add_table_str(lua_state(), "name", column.name().c_str());
         luaX_add_table_str(lua_state(), "type", column.type_name().c_str());
         luaX_add_table_str(lua_state(), "sql_type",
-                           column.sql_type_name(table.srid()).c_str());
+                           column.sql_type_name().c_str());
         luaX_add_table_str(lua_state(), "sql_modifiers",
                            column.sql_modifiers().c_str());
         luaX_add_table_bool(lua_state(), "not_null", column.not_null());
@@ -885,7 +922,7 @@ get_transform(lua_State *lua_state, flex_table_column_t const &column)
     char const *create_type = lua_tostring(lua_state, -1);
     if (create_type == nullptr) {
         throw std::runtime_error{
-            "Missing geometry transformation for column '{}'"_format(
+            "Missing geometry transformation for column '{}'."_format(
                 column.name())};
     }
 
@@ -895,7 +932,7 @@ get_transform(lua_State *lua_state, flex_table_column_t const &column)
     if (!transform->is_compatible_with(column.type())) {
         throw std::runtime_error{
             "Geometry transformation is not compatible "
-            "with column type '{}'"_format(column.type_name())};
+            "with column type '{}'."_format(column.type_name())};
     }
 
     lua_pop(lua_state, 1); // geom field
@@ -930,34 +967,40 @@ get_default_transform(flex_table_column_t const &column,
     }
 
     throw std::runtime_error{
-        "Missing geometry transformation for column '{}'"_format(
+        "Missing geometry transformation for column '{}'."_format(
             column.name())};
 }
 
 geom::osmium_builder_t::wkbs_t
-output_flex_t::run_transform(geom_transform_t const *transform,
+output_flex_t::run_transform(geom::osmium_builder_t *builder,
+                             geom_transform_t const *transform,
+                             table_column_type target_geom_type,
                              osmium::Node const &node)
 {
-    return transform->run(&m_builder, node);
+    return transform->run(builder, target_geom_type, node);
 }
 
 geom::osmium_builder_t::wkbs_t
-output_flex_t::run_transform(geom_transform_t const *transform,
+output_flex_t::run_transform(geom::osmium_builder_t *builder,
+                             geom_transform_t const *transform,
+                             table_column_type target_geom_type,
                              osmium::Way const & /*way*/)
 {
     if (get_way_nodes() <= 1U) {
         return {};
     }
-    return transform->run(&m_builder, m_context_way);
+    return transform->run(builder, target_geom_type, m_context_way);
 }
 
 geom::osmium_builder_t::wkbs_t
-output_flex_t::run_transform(geom_transform_t const *transform,
+output_flex_t::run_transform(geom::osmium_builder_t *builder,
+                             geom_transform_t const *transform,
+                             table_column_type target_geom_type,
                              osmium::Relation const &relation)
 {
     m_buffer.clear();
     auto const num_ways =
-        m_mid->rel_way_members_get(relation, nullptr, m_buffer);
+        m_mid->rel_way_members_get(relation, nullptr, &m_buffer);
 
     if (num_ways == 0) {
         return {};
@@ -967,7 +1010,7 @@ output_flex_t::run_transform(geom_transform_t const *transform,
         m_mid->nodes_get_list(&(way.nodes()));
     }
 
-    return transform->run(&m_builder, relation, m_buffer);
+    return transform->run(builder, target_geom_type, relation, m_buffer);
 }
 
 template <typename OBJECT>
@@ -980,10 +1023,18 @@ void output_flex_t::add_row(table_connection_t *table_connection,
     osmid_t const id = table.map_id(object.type(), object.id());
 
     if (!table.has_geom_column()) {
-        write_row(table_connection, object.type(), id, "");
+        write_row(table_connection, object.type(), id, "", 0);
         return;
     }
 
+    // From here we are handling the case where the table has a geometry
+    // column. In this case the second parameter to the Lua function add_row()
+    // must be present.
+    if (lua_gettop(lua_state()) == 0) {
+        throw std::runtime_error{
+            "Need two parameters: The osm2pgsql.table and the row data."};
+    }
+
     auto const geom_transform = get_transform(lua_state(), table.geom_column());
     assert(lua_gettop(lua_state()) == 1);
 
@@ -993,10 +1044,13 @@ void output_flex_t::add_row(table_connection_t *table_connection,
         transform = get_default_transform(table.geom_column(), object.type());
     }
 
-    auto const wkbs = run_transform(transform, object);
+    auto *builder = table_connection->get_builder();
+    auto const wkbs =
+        run_transform(builder, transform, table.geom_column().type(), object);
     for (auto const &wkb : wkbs) {
         m_expire.from_wkb(wkb.c_str(), id);
-        write_row(table_connection, object.type(), id, wkb);
+        write_row(table_connection, object.type(), id, wkb,
+                  table.geom_column().srid());
     }
 }
 
@@ -1014,7 +1068,7 @@ void output_flex_t::call_lua_function(prepared_lua_function_t func,
     if (luaX_pcall(lua_state(), 1, func.nresults())) {
         throw std::runtime_error{
             "Failed to execute Lua function 'osm2pgsql.{}':"
-            " {}"_format(func.name(), lua_tostring(lua_state(), -1))};
+            " {}."_format(func.name(), lua_tostring(lua_state(), -1))};
     }
 
     m_calling_context = calling_context::main;
@@ -1034,7 +1088,7 @@ void output_flex_t::pending_way(osmid_t id)
     }
 
     m_buffer.clear();
-    if (!m_mid->way_get(id, m_buffer)) {
+    if (!m_mid->way_get(id, &m_buffer)) {
         return;
     }
 
@@ -1069,7 +1123,7 @@ void output_flex_t::select_relation_members(osmium::Relation const &relation)
 
     if (lua_type(lua_state(), -1) != LUA_TTABLE) {
         throw std::runtime_error{"select_relation_members() returned something "
-                                 "other than nil or a table"};
+                                 "other than nil or a table."};
     }
 
     // We have established that we have a table. Get the 'ways' field...
@@ -1085,7 +1139,7 @@ void output_flex_t::select_relation_members(osmium::Relation const &relation)
     if (ltype != LUA_TTABLE) {
         throw std::runtime_error{
             "Table returned from select_relation_members() contains 'ways' "
-            "field, but it isn't an array table"};
+            "field, but it isn't an array table."};
     }
 
     // Iterate over the 'ways' table to get all ids...
@@ -1094,7 +1148,7 @@ void output_flex_t::select_relation_members(osmium::Relation const &relation)
         if (!lua_isnumber(lua_state(), -2)) {
             throw std::runtime_error{
                 "Table returned from select_relation_members() contains 'ways' "
-                "field, but it isn't an array table"};
+                "field, but it isn't an array table."};
         }
 
         osmid_t const id = lua_tointeger(lua_state(), -1);
@@ -1102,7 +1156,7 @@ void output_flex_t::select_relation_members(osmium::Relation const &relation)
             throw std::runtime_error{
                 "Table returned from select_relation_members() contains 'ways' "
                 "field, which must contain an array of non-zero integer way "
-                "ids"};
+                "ids."};
         }
 
         m_stage2_way_ids->set(id);
@@ -1118,7 +1172,7 @@ void output_flex_t::select_relation_members(osmid_t id)
         return;
     }
 
-    if (!m_mid->relation_get(id, m_rels_buffer)) {
+    if (!m_mid->relation_get(id, &m_rels_buffer)) {
         return;
     }
 
@@ -1133,7 +1187,7 @@ void output_flex_t::pending_relation(osmid_t id)
         return;
     }
 
-    if (!m_mid->relation_get(id, m_rels_buffer)) {
+    if (!m_mid->relation_get(id, &m_rels_buffer)) {
         return;
     }
     auto const &relation = m_rels_buffer.get<osmium::Relation>(0);
@@ -1156,7 +1210,7 @@ void output_flex_t::pending_relation_stage1c(osmid_t id)
         return;
     }
 
-    if (!m_mid->relation_get(id, m_rels_buffer)) {
+    if (!m_mid->relation_get(id, &m_rels_buffer)) {
         return;
     }
     auto const &relation = m_rels_buffer.get<osmium::Relation>(0);
@@ -1331,7 +1385,7 @@ output_flex_t::output_flex_t(
     std::shared_ptr<idset_t> stage2_way_ids)
 : output_t(mid, o), m_tables(std::move(tables)),
   m_stage2_way_ids(std::move(stage2_way_ids)), m_copy_thread(copy_thread),
-  m_lua_state(std::move(lua_state)), m_builder(o.projection),
+  m_lua_state(std::move(lua_state)),
   m_expire(o.expire_tiles_zoom, o.expire_tiles_max_bbox, o.projection),
   m_buffer(32768, osmium::memory::Buffer::auto_grow::yes),
   m_rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes),
@@ -1345,6 +1399,11 @@ output_flex_t::output_flex_t(
         init_lua(m_options.style);
     }
 
+    if (m_tables->empty()) {
+        throw std::runtime_error{
+            "No tables defined in Lua config. Nothing to do!"};
+    }
+
     assert(m_table_connections.empty());
     for (auto &table : *m_tables) {
         m_table_connections.emplace_back(&table, m_copy_thread);
@@ -1367,8 +1426,6 @@ void output_flex_t::init_lua(std::string const &filename)
     lua_newtable(lua_state());
 
     luaX_add_table_str(lua_state(), "version", get_osm2pgsql_short_version());
-    luaX_add_table_int(lua_state(), "srid",
-                       get_options()->projection->target_srs());
     luaX_add_table_str(lua_state(), "mode",
                        m_options.append ? "append" : "create");
     luaX_add_table_int(lua_state(), "stage", 1);
@@ -1380,7 +1437,7 @@ void output_flex_t::init_lua(std::string const &filename)
 
     // Define "osmpgsql.table" metatable
     if (luaL_newmetatable(lua_state(), osm2pgsql_table_name) != 1) {
-        throw std::runtime_error{"Internal error: Lua newmetatable failed"};
+        throw std::runtime_error{"Internal error: Lua newmetatable failed."};
     }
     lua_pushvalue(lua_state(), -1);
     lua_setfield(lua_state(), -2, "__index");
@@ -1396,7 +1453,7 @@ void output_flex_t::init_lua(std::string const &filename)
 
     // Load compiled in init.lua
     if (luaL_dostring(lua_state(), lua_init())) {
-        throw std::runtime_error{"Internal error in Lua setup: {}"_format(
+        throw std::runtime_error{"Internal error in Lua setup: {}."_format(
             lua_tostring(lua_state(), -1))};
     }
 
@@ -1418,7 +1475,7 @@ void output_flex_t::init_lua(std::string const &filename)
     // Load user config file
     luaX_set_context(lua_state(), this);
     if (luaL_dofile(lua_state(), filename.c_str())) {
-        throw std::runtime_error{"Error loading lua config: {}"_format(
+        throw std::runtime_error{"Error loading lua config: {}."_format(
             lua_tostring(lua_state(), -1))};
     }
 
@@ -1442,11 +1499,10 @@ void output_flex_t::init_lua(std::string const &filename)
 idset_t const &output_flex_t::get_marked_way_ids()
 {
     if (m_stage2_way_ids->empty()) {
-        fmt::print(stderr, "Skipping stage 1c (no marked ways).\n");
+        log_info("Skipping stage 1c (no marked ways).");
     } else {
-        fmt::print(stderr,
-                   "Entering stage 1c processing of {} ways...\n"_format(
-                       m_stage2_way_ids->size()));
+        log_info("Entering stage 1c processing of {} ways..."_format(
+            m_stage2_way_ids->size()));
         m_stage2_way_ids->sort_unique();
     }
 
@@ -1456,44 +1512,41 @@ idset_t const &output_flex_t::get_marked_way_ids()
 void output_flex_t::reprocess_marked()
 {
     if (m_stage2_way_ids->empty()) {
-        fmt::print(stderr, "No marked ways (Skipping stage 2).\n");
+        log_info("No marked ways (Skipping stage 2).");
         return;
     }
 
-    fmt::print(stderr, "Reprocess marked ways (stage 2)...\n");
+    log_info("Reprocess marked ways (stage 2)...");
 
     if (!m_options.append) {
-        fmt::print(stderr, "Creating id indexes...\n");
         util::timer_t timer;
 
         for (auto &table : m_table_connections) {
             if (table.table().matches_type(osmium::item_type::way)) {
-                fmt::print(stderr, "  Creating id index on table '{}'...\n",
-                           table.table().name());
                 table.create_id_index();
             }
         }
 
-        fmt::print(stderr, "  Creating id indexes took {} seconds\n"_format(
-                               timer.stop()));
+        log_info("Creating id indexes took {}"_format(
+            util::human_readable_duration(timer.stop())));
     }
 
     lua_gc(lua_state(), LUA_GCCOLLECT, 0);
-    fmt::print(stderr, "Lua program uses {} MBytes\n",
-               lua_gc(lua_state(), LUA_GCCOUNT, 0) / 1024);
+    log_debug("Lua program uses {} MBytes",
+              lua_gc(lua_state(), LUA_GCCOUNT, 0) / 1024);
 
     lua_getglobal(lua_state(), "osm2pgsql");
     lua_pushinteger(lua_state(), 2);
     lua_setfield(lua_state(), -2, "stage");
     lua_pop(lua_state(), 1); // osm2pgsql
 
-    fmt::print(stderr, "There are {} ways to reprocess...\n"_format(
-                           m_stage2_way_ids->size()));
+    log_info(
+        "There are {} ways to reprocess..."_format(m_stage2_way_ids->size()));
 
     m_stage2_way_ids->sort_unique();
     for (osmid_t const id : *m_stage2_way_ids) {
         m_buffer.clear();
-        if (!m_mid->way_get(id, m_buffer)) {
+        if (!m_mid->way_get(id, &m_buffer)) {
             continue;
         }
         auto &way = m_buffer.get<osmium::Way>(0);
