@@ -1,6 +1,15 @@
 #ifndef OSM2PGSQL_OUTPUT_GAZETTEER_HPP
 #define OSM2PGSQL_OUTPUT_GAZETTEER_HPP
 
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <memory>
 
 #include <osmium/memory/buffer.hpp>
@@ -10,7 +19,6 @@
 #include "osmium-builder.hpp"
 #include "osmtypes.hpp"
 #include "output.hpp"
-#include "util.hpp"
 
 class output_gazetteer_t : public output_t
 {
@@ -45,8 +53,6 @@ public:
     void stop(thread_pool_t *) noexcept override {}
     void sync() override;
 
-    bool need_forward_dependencies() const noexcept override { return false; }
-
     void pending_way(osmid_t) noexcept override {}
     void pending_relation(osmid_t) noexcept override {}
 
