@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 /* Implements the mid-layer processing for osm2pgsql
  * using several PostgreSQL tables
  *
@@ -21,6 +30,7 @@
 #include <unistd.h>
 
 #include "expire-tiles.hpp"
+#include "logging.hpp"
 #include "middle.hpp"
 #include "node-ram-cache.hpp"
 #include "options.hpp"
@@ -70,12 +80,12 @@ void output_pgsql_t::pending_way(osmid_t id)
 {
     // Try to fetch the way from the DB
     buffer.clear();
-    if (m_mid->way_get(id, buffer)) {
+    if (m_mid->way_get(id, &buffer)) {
         pgsql_delete_way_from_output(id);
 
         taglist_t outtags;
-        int polygon;
-        int roads;
+        int polygon = 0;
+        int roads = 0;
         auto &way = buffer.get<osmium::Way>(0);
         if (!m_tagtransform->filter_tags(way, &polygon, &roads, outtags)) {
             auto nnodes = m_mid->nodes_get_list(&(way.nodes()));
@@ -94,7 +104,7 @@ void output_pgsql_t::pending_relation(osmid_t id)
     // we cannot keep a reference to the relation and an autogrow buffer
     // might be relocated when more data is added.
     rels_buffer.clear();
-    if (m_mid->relation_get(id, rels_buffer)) {
+    if (m_mid->relation_get(id, &rels_buffer)) {
         pgsql_delete_relation_from_output(id);
 
         auto const &rel = rels_buffer.get<osmium::Relation>(0);
@@ -165,7 +175,7 @@ void output_pgsql_t::pgsql_process_relation(osmium::Relation const &rel)
 
     buffer.clear();
     rolelist_t xrole;
-    auto num_ways = m_mid->rel_way_members_get(rel, &xrole, buffer);
+    auto num_ways = m_mid->rel_way_members_get(rel, &xrole, &buffer);
 
     if (num_ways == 0) {
         return;
@@ -343,6 +353,9 @@ output_pgsql_t::output_pgsql_t(
   buffer(32768, osmium::memory::Buffer::auto_grow::yes),
   rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
 {
+    log_debug("Using projection SRS {} ({})", o.projection->target_srs(),
+              o.projection->target_desc());
+
     export_list exlist;
 
     m_enable_way_area = read_style_file(m_options.style, &exlist);
@@ -378,14 +391,14 @@ output_pgsql_t::output_pgsql_t(
             type = "LINESTRING";
             break;
         default:
-            //TODO: error message about coding error
-            util::exit_nicely();
+            std::abort(); // should never be here
         }
 
         m_tables[i].reset(
             new table_t{name, type, columns, m_options.hstore_columns,
                         m_options.projection->target_srs(), m_options.append,
-                        m_options.hstore_mode, copy_thread});
+                        m_options.hstore_mode, copy_thread,
+                        m_options.output_dbschema});
     }
 }
 
