@@ -1,5 +1,14 @@
-#ifndef OSM2PGSQL_TEST_COMMON_IMPORT_HPP
-#define OSM2PGSQL_TEST_COMMON_IMPORT_HPP
+#ifndef OSM2PGSQL_TESTS_COMMON_IMPORT_HPP
+#define OSM2PGSQL_TESTS_COMMON_IMPORT_HPP
+
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
 
 #include <osmium/handler.hpp>
 #include <osmium/io/any_input.hpp>
@@ -10,6 +19,7 @@
 
 #include "dependency-manager.hpp"
 #include "geometry-processor.hpp"
+#include "input.hpp"
 #include "middle-pgsql.hpp"
 #include "middle-ram.hpp"
 #include "osmdata.hpp"
@@ -30,7 +40,8 @@ inline void parse_file(options_t const &options,
                        std::unique_ptr<dependency_manager_t> dependency_manager,
                        std::shared_ptr<middle_t> const &mid,
                        std::vector<std::shared_ptr<output_t>> const &outs,
-                       char const *filename = nullptr)
+                       char const *filename = nullptr,
+                       bool do_stop = true)
 {
     osmdata_t osmdata{std::move(dependency_manager), mid, outs, options};
 
@@ -42,12 +53,16 @@ inline void parse_file(options_t const &options,
     } else {
         filepath += options.input_files[0];
     }
-    osmium::io::File file{filepath};
-    osmdata.process_file(file, options.bbox);
+    osmium::io::File const file{filepath};
+    process_files({file}, &osmdata, options.append, false);
 
-    osmdata.stop();
+    if (do_stop) {
+        osmdata.stop();
+    }
 }
 
+namespace db {
+
 /**
  * This is used as a helper to assemble OSM objects into an OPL file which
  * can later be used as input for testing.
@@ -111,8 +126,6 @@ private:
     std::string m_result;
 };
 
-namespace db {
-
 /**
  * Convenience class around tempdb_t that offers functions for
  * data import from file and strings.
@@ -120,7 +133,8 @@ namespace db {
 class import_t
 {
 public:
-    void run_import(options_t options, char const *data,
+    void run_import(options_t options,
+                    std::initializer_list<std::string> input_data,
                     std::string const &format = "opl")
     {
         options.database_options = m_db.db_options();
@@ -137,27 +151,31 @@ public:
         auto const outputs =
             output_t::create_outputs(middle->get_query_instance(), options);
 
-        bool const need_dependencies =
-            std::any_of(outputs.cbegin(), outputs.cend(),
-                        [](std::shared_ptr<output_t> const &output) {
-                            return output->need_forward_dependencies();
-                        });
-
         auto dependency_manager = std::unique_ptr<dependency_manager_t>(
-            need_dependencies ? new full_dependency_manager_t{middle}
-                              : new dependency_manager_t{});
+            options.with_forward_dependencies
+                ? new full_dependency_manager_t{middle}
+                : new dependency_manager_t{});
 
         osmdata_t osmdata{std::move(dependency_manager), middle, outputs,
                           options};
 
         osmdata.start();
 
-        osmium::io::File file{data, std::strlen(data), format};
-        osmdata.process_file(file, options.bbox);
+        std::vector<osmium::io::File> files;
+        for (auto const &data : input_data) {
+            files.emplace_back(data.data(), data.size(), format);
+        }
+        process_files(files, &osmdata, options.append, false);
 
         osmdata.stop();
     }
 
+    void run_import(options_t options, char const *data,
+                    std::string const &format = "opl")
+    {
+        run_import(options, std::initializer_list<std::string>{data}, format);
+    }
+
     void run_file(options_t options, char const *file = nullptr)
     {
         options.database_options = m_db.db_options();
@@ -208,15 +226,15 @@ public:
                    {out_test}, file);
     }
 
-    pg::conn_t connect() { return m_db.connect(); }
+    testing::pg::conn_t connect() { return m_db.connect(); }
 
-    pg::tempdb_t const &db() const { return m_db; }
+    testing::pg::tempdb_t const &db() const { return m_db; }
 
 private:
-    pg::tempdb_t m_db;
+    testing::pg::tempdb_t m_db;
 };
 
 } // namespace db
 } // namespace testing
 
-#endif // OSM2PGSQL_TEST_COMMON_IMPORT_HPP
+#endif // OSM2PGSQL_TESTS_COMMON_IMPORT_HPP
