@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <catch.hpp>
 
 #include <vector>
@@ -9,11 +18,12 @@ static testing::db::import_t db;
 
 TEST_CASE("Projection setup")
 {
-    std::vector<char const *> option_params = {
-        "osm2pgsql", "-S", OSM2PGSQLDATA_DIR "default.style",
-        "--number-processes", "1"};
+    char const* const style_file = OSM2PGSQLDATA_DIR "default.style";
 
-    char const *proj_name = nullptr;
+    std::vector<char const *> option_params = {"osm2pgsql", "-S", style_file,
+                                               "--number-processes", "1"};
+
+    std::string proj_name;
     char const *srid = "";
 
     SECTION("No options")
@@ -29,7 +39,7 @@ TEST_CASE("Projection setup")
         srid = "4326";
     }
 
-    SECTION("Mercartor option")
+    SECTION("Mercator option")
     {
         option_params.push_back("-m");
         proj_name = "Spherical Mercator";
@@ -63,10 +73,10 @@ TEST_CASE("Projection setup")
 
     option_params.push_back("foo");
 
-    options_t options((int)option_params.size(), (char **)option_params.data());
+    options_t options{(int)option_params.size(), (char **)option_params.data()};
 
-    if (proj_name) {
-        CHECK(strcmp(options.projection->target_desc(), proj_name) == 0);
+    if (!proj_name.empty()) {
+        CHECK(options.projection->target_desc() == proj_name);
     }
 
     db.run_import(options, "n1 Tamenity=bar x0 y0");
@@ -74,5 +84,5 @@ TEST_CASE("Projection setup")
     auto conn = db.connect();
 
     CHECK(conn.require_scalar<std::string>(
-              "SELECT find_srid('public', 'planet_osm_roads', 'way')") == srid);
+              "SELECT Find_SRID('public', 'planet_osm_roads', 'way')") == srid);
 }
