@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <catch.hpp>
 
 #include "common-import.hpp"
@@ -9,8 +18,7 @@ static char const *const conf_file = "test_output_flex_line.lua";
 
 TEST_CASE("linestring in latlon projection (unsplit and split)")
 {
-    options_t const options =
-        testing::opt_t().flex(conf_file).srs(PROJ_LATLONG);
+    options_t const options = testing::opt_t().flex(conf_file);
 
     REQUIRE_NOTHROW(db.run_import(options,
                                   "n10 v1 dV x1.0 y1.0\n"
