@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <catch.hpp>
 
 #include "common-import.hpp"
@@ -9,8 +18,7 @@ static char const *const conf_file = "test_output_flex_stage2.lua";
 
 TEST_CASE("nodes and ways")
 {
-    options_t options =
-        testing::opt_t().slim().flex(conf_file).srs(PROJ_LATLONG);
+    options_t options = testing::opt_t().slim().flex(conf_file);
 
     REQUIRE_NOTHROW(db.run_import(options,
                                   "n10 v1 dV x10.0 y10.0\n"
@@ -88,8 +96,7 @@ TEST_CASE("nodes and ways")
 
 TEST_CASE("relation data on ways")
 {
-    options_t options =
-        testing::opt_t().slim().flex(conf_file).srs(PROJ_LATLONG);
+    options_t options = testing::opt_t().slim().flex(conf_file);
 
     // create database with three ways and a relation on two of them
     REQUIRE_NOTHROW(
@@ -187,8 +194,7 @@ TEST_CASE("relation data on ways")
 
 TEST_CASE("relation data on ways: delete or re-tag relation")
 {
-    options_t options =
-        testing::opt_t().slim().flex(conf_file).srs(PROJ_LATLONG);
+    options_t options = testing::opt_t().slim().flex(conf_file);
 
     // create database with three ways and a relation on two of them
     REQUIRE_NOTHROW(
@@ -247,8 +253,7 @@ TEST_CASE("relation data on ways: delete or re-tag relation")
 
 TEST_CASE("relation data on ways: delete way in other relation")
 {
-    options_t options =
-        testing::opt_t().slim().flex(conf_file).srs(PROJ_LATLONG);
+    options_t options = testing::opt_t().slim().flex(conf_file);
 
     // create database with three ways and two relations on them
     REQUIRE_NOTHROW(
@@ -317,8 +322,7 @@ TEST_CASE("relation data on ways: delete way in other relation")
 TEST_CASE("relation data on ways: changing things in one relation should not "
           "change output")
 {
-    options_t options =
-        testing::opt_t().slim().flex(conf_file).srs(PROJ_LATLONG);
+    options_t options = testing::opt_t().slim().flex(conf_file);
 
     // create database with three ways and two relations on them
     REQUIRE_NOTHROW(
@@ -394,8 +398,7 @@ TEST_CASE("relation data on ways: changing things in one relation should not "
 
 TEST_CASE("relation data on ways: change relation (two rels)")
 {
-    options_t options =
-        testing::opt_t().slim().flex(conf_file).srs(PROJ_LATLONG);
+    options_t options = testing::opt_t().slim().flex(conf_file);
 
     // create database with three ways and two relations on them
     REQUIRE_NOTHROW(
@@ -451,8 +454,7 @@ TEST_CASE("relation data on ways: change relation (two rels)")
 
 TEST_CASE("relation data on ways: change relation (three rels)")
 {
-    options_t options =
-        testing::opt_t().slim().flex(conf_file).srs(PROJ_LATLONG);
+    options_t options = testing::opt_t().slim().flex(conf_file);
 
     // create database with three ways and two relations on them
     REQUIRE_NOTHROW(db.run_import(options,
@@ -526,10 +528,8 @@ TEST_CASE("relation data on ways: change relation (three rels)")
 
 TEST_CASE("relation data on ways: delete relation")
 {
-    options_t options = testing::opt_t()
-                            .slim()
-                            .flex("test_output_flex_stage2_alt.lua")
-                            .srs(PROJ_LATLONG);
+    options_t options =
+        testing::opt_t().slim().flex("test_output_flex_stage2_alt.lua");
 
     // create database with a way and two relations on it
     REQUIRE_NOTHROW(db.run_import(options,
