@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <catch.hpp>
 
 #include <random>
@@ -121,7 +130,7 @@ public:
                        id);
     }
 
-    void del(osmid_t id) { fmt::format_to(m_way_opl, "r{} v2 dD\n", id); }
+    void del(osmid_t id) { fmt::format_to(m_rel_opl, "r{} v2 dD\n", id); }
 
     std::string get_and_clear_opl()
     {
@@ -191,7 +200,8 @@ public:
         REQUIRE_NOTHROW(db.run_import(opt, opl.c_str()));
     }
 
-    unsigned long obj_count(pg::conn_t const &conn, osmid_t id, char const *cls)
+    unsigned long obj_count(testing::pg::conn_t const &conn, osmid_t id,
+                            char const *cls)
     {
         char const tchar = m_opl_factory.type();
         return conn.get_count("place",
@@ -200,26 +210,26 @@ public:
                               "AND class = '{}'"_format(tchar, id, cls));
     }
 
-    void obj_names(pg::conn_t const &conn, osmid_t id, char const *cls,
+    void obj_names(testing::pg::conn_t const &conn, osmid_t id, char const *cls,
                    hstore_list const &names)
     {
         hstore_compare(conn, id, cls, "name", names);
     }
 
-    void obj_address(pg::conn_t const &conn, osmid_t id, char const *cls,
-                     hstore_list const &names)
+    void obj_address(testing::pg::conn_t const &conn, osmid_t id,
+                     char const *cls, hstore_list const &names)
     {
         hstore_compare(conn, id, cls, "address", names);
     }
 
-    void obj_extratags(pg::conn_t const &conn, osmid_t id, char const *cls,
-                       hstore_list const &names)
+    void obj_extratags(testing::pg::conn_t const &conn, osmid_t id,
+                       char const *cls, hstore_list const &names)
     {
         hstore_compare(conn, id, cls, "extratags", names);
     }
 
-    std::string obj_field(pg::conn_t const &conn, osmid_t id, char const *cls,
-                          char const *column)
+    std::string obj_field(testing::pg::conn_t const &conn, osmid_t id,
+                          char const *cls, char const *column)
     {
         char const tchar = m_opl_factory.type();
         return conn.require_scalar<std::string>(
@@ -228,8 +238,9 @@ public:
     }
 
 private:
-    void hstore_compare(pg::conn_t const &conn, osmid_t id, char const *cls,
-                        char const *column, hstore_list const &names)
+    void hstore_compare(testing::pg::conn_t const &conn, osmid_t id,
+                        char const *cls, char const *column,
+                        hstore_list const &names)
     {
         char const tchar = m_opl_factory.type();
         auto const sql =
@@ -303,9 +314,9 @@ TEMPLATE_TEST_CASE("Main tags", "", node_importer_t, way_importer_t,
         CHECK(1 == t.obj_count(conn, 2, "railway"));
         CHECK(1 == t.obj_count(conn, 3, "amenity"));
 
-        t.del(3);
         t.add(1, "not_a=restaurant");
         t.add(2, "highway=bus_stop,name=X");
+        t.del(3);
 
         t.update();
 
