@@ -1,3 +1,12 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <catch.hpp>
 
 #include "common-import.hpp"
@@ -5,7 +14,7 @@
 
 static testing::db::import_t db;
 
-static void require_tables(pg::conn_t const &conn)
+static void require_tables(testing::pg::conn_t const &conn)
 {
     conn.require_has_table("osm2pgsql_test_point");
     conn.require_has_table("osm2pgsql_test_line");
