@@ -1,10 +1,17 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
+
 #include <catch.hpp>
 
 #include "node-persistent-cache.hpp"
-#include "options.hpp"
 
 #include "common-cleanup.hpp"
-#include "common-options.hpp"
 
 static void write_and_read_location(node_persistent_cache &cache, osmid_t id,
                                     double x, double y)
@@ -27,13 +34,12 @@ static void delete_location(node_persistent_cache &cache, osmid_t id)
 
 TEST_CASE("Persistent cache", "[NoDB]")
 {
-    options_t const options = testing::opt_t().flatnodes();
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file.get()};
-    auto ram_cache = std::make_shared<node_ram_cache>(); // dummy cache
+    std::string const flat_node_file = "test_middle_flat.flat.nodes.bin";
+    testing::cleanup::file_t flatnode_cleaner{flat_node_file};
 
     // create a new cache
     {
-        node_persistent_cache cache{&options, ram_cache};
+        node_persistent_cache cache{flat_node_file, false};
 
         // write in order
         write_and_read_location(cache, 10, 10.01, -45.3);
@@ -56,7 +62,7 @@ TEST_CASE("Persistent cache", "[NoDB]")
 
     // reopen the cache
     {
-        node_persistent_cache cache{&options, ram_cache};
+        node_persistent_cache cache{flat_node_file, false};
 
         // read all previously written locations
         read_location(cache, 10, 10.01, -45.3);
