@@ -1,9 +1,17 @@
+/**
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This file is part of osm2pgsql (https://osm2pgsql.org/).
+ *
+ * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * For a full list of authors see the git log.
+ */
 
 #include <catch.hpp>
 
 #include "taginfo-impl.hpp"
 
-TEST_CASE("parse_tag_flags")
+TEST_CASE("parse_tag_flags", "[NoDB]")
 {
     CHECK(parse_tag_flags("", 0) == 0);
     CHECK(parse_tag_flags("polygon", 0) == FLAG_POLYGON);
